@@ -1,10 +1,3 @@
-"""
-usage examples:
-python -m tmrl --server
-python -m tmrl --trainer
-python -m tmrl --worker
-"""
-
 import time
 from argparse import ArgumentParser, ArgumentTypeError
 import logging
