@@ -70,16 +70,15 @@ def replace_hist_before_done(hist, done_idx_in_hist):
 
 class MemoryTMNF(MemoryDataloading):
     def __init__(self,
-                 memory_size,
-                 batchsize,
-                 path_loc="",
+                 memory_size=None,
+                 batch_size=None,
+                 dataset_path="",
                  imgs_obs=4,
                  act_buf_len=1,
                  nb_steps=1,
                  use_dataloader=False,
                  num_workers=0,
                  pin_memory=False,
-                 remove_size=100,
                  obs_preprocessor: callable = None,
                  sample_preprocessor: callable = None,
                  crc_debug=False,
@@ -90,13 +89,12 @@ class MemoryTMNF(MemoryDataloading):
         self.start_imgs_offset = max(0, self.min_samples - self.imgs_obs)
         self.start_acts_offset = max(0, self.min_samples - self.act_buf_len)
         super().__init__(memory_size=memory_size,
-                         batchsize=batchsize,
-                         path_loc=path_loc,
+                         batch_size=batch_size,
+                         dataset_path=dataset_path,
                          nb_steps=nb_steps,
                          use_dataloader=use_dataloader,
                          num_workers=num_workers,
                          pin_memory=pin_memory,
-                         remove_size=remove_size,
                          obs_preprocessor=obs_preprocessor,
                          sample_preprocessor=sample_preprocessor,
                          crc_debug=crc_debug,
@@ -221,9 +219,9 @@ class MemoryTMNFLidar(MemoryTMNF):
 
 class TrajMemoryTMNF(TrajMemoryDataloading):
     def __init__(self,
-                 memory_size,
-                 batchsize,
-                 path_loc="",
+                 memory_size=None,
+                 batch_size=None,
+                 dataset_path="",
                  imgs_obs=4,
                  act_buf_len=1,
                  traj_len=1,
@@ -231,7 +229,6 @@ class TrajMemoryTMNF(TrajMemoryDataloading):
                  use_dataloader=False,
                  num_workers=0,
                  pin_memory=False,
-                 remove_size=100,
                  obs_preprocessor: callable = None,
                  crc_debug=False,
                  device="cpu"):
@@ -243,13 +240,12 @@ class TrajMemoryTMNF(TrajMemoryDataloading):
         self.start_imgs_offset = max(0, self.min_samples - self.imgs_obs)
         self.start_acts_offset = max(0, self.min_samples - self.act_buf_len)
         super().__init__(memory_size=memory_size,
-                         batchsize=batchsize,
-                         path_loc=path_loc,
+                         batch_size=batch_size,
+                         dataset_path=dataset_path,
                          nb_steps=nb_steps,
                          use_dataloader=use_dataloader,
                          num_workers=num_workers,
                          pin_memory=pin_memory,
-                         remove_size=remove_size,
                          obs_preprocessor=obs_preprocessor,
                          crc_debug=crc_debug,
                          device=device)
@@ -362,16 +358,15 @@ class TrajMemoryTMNFLidar(TrajMemoryTMNF):
 
 class MemoryTM2020(MemoryDataloading):  # TODO: reset transitions
     def __init__(self,
-                 memory_size,
-                 batchsize,
-                 path_loc="",
+                 memory_size=None,
+                 batch_size=None,
+                 dataset_path="",
                  imgs_obs=4,
                  act_buf_len=1,
                  nb_steps=1,
                  use_dataloader=False,
                  num_workers=0,
                  pin_memory=False,
-                 remove_size=100,
                  obs_preprocessor: callable = None,
                  sample_preprocessor: callable = None,
                  crc_debug=False,
@@ -382,13 +377,12 @@ class MemoryTM2020(MemoryDataloading):  # TODO: reset transitions
         self.start_imgs_offset = max(0, self.min_samples - self.imgs_obs)
         self.start_acts_offset = max(0, self.min_samples - self.act_buf_len)
         super().__init__(memory_size=memory_size,
-                         batchsize=batchsize,
-                         path_loc=path_loc,
+                         batch_size=batch_size,
+                         dataset_path=dataset_path,
                          nb_steps=nb_steps,
                          use_dataloader=use_dataloader,
                          num_workers=num_workers,
                          pin_memory=pin_memory,
-                         remove_size=remove_size,
                          obs_preprocessor=obs_preprocessor,
                          sample_preprocessor=sample_preprocessor,
                          crc_debug=crc_debug,
